@@ -32,7 +32,7 @@
 //! ```
 
 #![cfg_attr(has_extern_crate_alloc, no_std)]
-#![doc(html_root_url = "https://docs.rs/circular-queue/0.2.5")]
+#![doc(html_root_url = "https://docs.rs/circular-queue/0.2.6")]
 
 #[cfg(has_extern_crate_alloc)]
 extern crate alloc;
@@ -42,11 +42,15 @@ use alloc::vec::Vec;
 #[cfg(has_extern_crate_alloc)]
 use core::iter::{Chain, Rev};
 #[cfg(has_extern_crate_alloc)]
+use core::mem::replace;
+#[cfg(has_extern_crate_alloc)]
 use core::slice::{Iter as SliceIter, IterMut as SliceIterMut};
 
 #[cfg(not(has_extern_crate_alloc))]
 use std::iter::{Chain, Rev};
 #[cfg(not(has_extern_crate_alloc))]
+use std::mem::replace;
+#[cfg(not(has_extern_crate_alloc))]
 use std::slice::{Iter as SliceIter, IterMut as SliceIterMut};
 
 #[cfg(feature = "serde_support")]
@@ -74,6 +78,9 @@ pub type AscIter<'a, T> = Chain<SliceIter<'a, T>, SliceIter<'a, T>>;
 /// An mutable ascending iterator over `CircularQueue<T>`.
 pub type AscIterMut<'a, T> = Chain<SliceIterMut<'a, T>, SliceIterMut<'a, T>>;
 
+/// A value popped from `CircularQueue<T>` as the result of a push operation.
+pub type Popped<T> = Option<T>;
+
 impl<T> CircularQueue<T> {
     /// Constructs a new, empty `CircularQueue<T>` with the requested capacity.
     ///
@@ -130,6 +137,30 @@ impl<T> CircularQueue<T> {
         self.data.is_empty()
     }
 
+    /// Returns `true` if the queue is full.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use circular_queue::CircularQueue;
+    ///
+    /// let mut queue = CircularQueue::with_capacity(5);
+    ///
+    /// assert!(!queue.is_full());
+    ///
+    /// queue.push(1);
+    /// queue.push(2);
+    /// queue.push(3);
+    /// queue.push(4);
+    /// queue.push(5);
+    ///
+    /// assert!(queue.is_full());
+    /// ```
+    #[inline]
+    pub fn is_full(&self) -> bool {
+        self.capacity() == self.len()
+    }
+
     /// Returns the capacity of the queue.
     ///
     /// # Examples
@@ -170,16 +201,20 @@ impl<T> CircularQueue<T> {
     ///
     /// Once the capacity is reached, pushing new items will overwrite old ones.
     ///
+    /// In case an old value is overwritten, it will be returned.
+    ///
     /// # Examples
     ///
     /// ```
     /// use circular_queue::CircularQueue;
     ///
     /// let mut queue = CircularQueue::with_capacity(3);
+    ///
     /// queue.push(1);
     /// queue.push(2);
-    /// queue.push(3);
-    /// queue.push(4);
+    ///
+    /// assert_eq!(queue.push(3), None);
+    /// assert_eq!(queue.push(4), Some(1));
     ///
     /// assert_eq!(queue.len(), 3);
     ///
@@ -190,18 +225,22 @@ impl<T> CircularQueue<T> {
     /// assert_eq!(iter.next(), Some(&2));
     /// ```
     #[inline]
-    pub fn push(&mut self, x: T) {
+    pub fn push(&mut self, x: T) -> Popped<T> {
+        let mut old = None;
+
         if self.capacity() == 0 {
-            return;
+            return old;
         }
 
-        if self.data.len() < self.capacity() {
+        if !self.is_full() {
             self.data.push(x);
         } else {
-            self.data[self.insertion_index] = x;
+            old = Some(replace(&mut self.data[self.insertion_index], x));
         }
 
         self.insertion_index = (self.insertion_index + 1) % self.capacity();
+
+        old
     }
 
     /// Returns an iterator over the queue's contents.
